@@ -1,43 +1,81 @@
 //
-// "$Id: ExternalCodeEditor_WIN32.cxx 11879 2016-08-16 21:08:40Z greg.ercolano $".
+// External code editor management class for Windows
 //
-//       External code editor management class for Windows
+// Copyright 1998-2023 by Bill Spitzak and others.
+//
+// This library is free software. Distribution and use rights are outlined in
+// the file "COPYING" which should have been included with this file.  If this
+// file is missing or damaged, see the license at:
+//
+//     https://www.fltk.org/COPYING.php
+//
+// Please see the following page on how to report bugs and issues:
+//
+//     https://www.fltk.org/bugs.php
 //
 
-#ifdef WIN32	        /* This entire file windows only */
-
-#include <stdio.h>      // snprintf()
+//      Note: This entire file Windows only.
 
 #include <FL/Fl.H>      // Fl_Timeout_Handler..
 #include <FL/fl_ask.H>  // fl_alert()
+#include <FL/fl_utf8.h> // fl_utf8fromwc()
+#include <FL/fl_string_functions.h> // fl_strdup()
 
 #include "ExternalCodeEditor_WIN32.h"
+#include "fluid.h"
+
+#include <stdio.h>      // snprintf()
+#include <stdlib.h>
 
 extern int G_debug;     // defined in fluid.cxx
 
 // Static local data
 static int L_editors_open = 0;                          // keep track of #editors open
 static Fl_Timeout_Handler L_update_timer_cb = 0;        // app's update timer callback
+static wchar_t *wbuf = NULL;
+static char *abuf = NULL;
+
+static wchar_t *utf8_to_wchar(const char *utf8, wchar_t *&wbuf, int lg = -1) {
+  unsigned len = (lg >= 0) ? (unsigned)lg : (unsigned)strlen(utf8);
+  unsigned wn = fl_utf8toUtf16(utf8, len, NULL, 0) + 1; // Query length
+  wbuf = (wchar_t *)realloc(wbuf, sizeof(wchar_t) * wn);
+  wn = fl_utf8toUtf16(utf8, len, (unsigned short *)wbuf, wn); // Convert string
+  wbuf[wn] = 0;
+  return wbuf;
+}
+
+static char *wchar_to_utf8(const wchar_t *wstr, char *&utf8) {
+  unsigned len = (unsigned)wcslen(wstr);
+  unsigned wn = fl_utf8fromwc(NULL, 0, wstr, len) + 1; // query length
+  utf8 = (char *)realloc(utf8, wn);
+  wn = fl_utf8fromwc(utf8, wn, wstr, len); // convert string
+  utf8[wn] = 0;
+  return utf8;
+}
 
 // [Static/Local] Get error message string for last failed WIN32 function.
 // Returns a string pointing to static memory.
 //
-//     TODO: Is more code needed here to convert returned string to utf8? -erco
-//
 static const char *get_ms_errmsg() {
   static char emsg[1024];
   DWORD lastErr = GetLastError();
   DWORD flags = FORMAT_MESSAGE_ALLOCATE_BUFFER |
                 FORMAT_MESSAGE_IGNORE_INSERTS  |
                 FORMAT_MESSAGE_FROM_SYSTEM;
-  LPSTR mbuf = 0;
-  DWORD size = FormatMessageA(flags, 0, lastErr, MAKELANGID(LANG_NEUTRAL,
-                              SUBLANG_DEFAULT), (LPSTR)&mbuf, 0, NULL);
-  if ( size == 0 ) {
-    _snprintf(emsg, sizeof(emsg), "Error Code %ld", long(lastErr));
+  DWORD langid = MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT);
+  LPWSTR mbuf = 0;
+  DWORD msize = 0;
+
+  // Get error message from Windows
+  msize = FormatMessageW(flags, 0, lastErr, langid, (LPWSTR)&mbuf, 0, NULL);
+  if ( msize == 0 ) {
+    _snprintf(emsg, sizeof(emsg), "Error #%ld", (unsigned long)lastErr);
   } else {
-    // Copy mbuf -> emsg (with '\r's removed -- they screw up fl_alert())
-    for ( char *src=mbuf, *dst=emsg; 1; src++ ) {
+    // Convert message to UTF-8
+    fl_utf8fromwc(emsg, sizeof(emsg), mbuf, msize);
+    // Remove '\r's -- they screw up fl_alert()
+    char *src = emsg, *dst = emsg;
+    for ( ; 1; src++ ) {
       if ( *src == '\0' ) { *dst = '\0'; break; }
       if ( *src != '\r' ) { *dst++ = *src; }
     }
@@ -48,15 +86,18 @@ static const char *get_ms_errmsg() {
 
 // [Static/Local] See if file exists
 static int is_file(const char *filename) {
-  DWORD att = GetFileAttributesA(filename);
-  if (att == INVALID_FILE_ATTRIBUTES) return 0;
+  utf8_to_wchar(filename, wbuf);
+  DWORD att = GetFileAttributesW(wbuf);
+  if (att == INVALID_FILE_ATTRIBUTES)
+    return 0;
   if ( (att & FILE_ATTRIBUTE_DIRECTORY) == 0 ) return 1;        // not a dir == file
   return 0;
 }
 
 // [Static/Local] See if dir exists
 static int is_dir(const char *dirname) {
-  DWORD att = GetFileAttributesA(dirname);
+  utf8_to_wchar(dirname, wbuf);
+  DWORD att = GetFileAttributesW(wbuf);
   if (att == INVALID_FILE_ATTRIBUTES) return 0;
   if (att & FILE_ATTRIBUTE_DIRECTORY) return 1;
   return 0;
@@ -81,7 +122,7 @@ ExternalCodeEditor::~ExternalCodeEditor() {
 //
 void ExternalCodeEditor::set_filename(const char *val) {
   if ( filename_ ) free((void*)filename_);
-  filename_ = val ? strdup(val) : 0;
+  filename_ = val ? fl_strdup(val) : 0;
 }
 
 // [Public] Is editor running?
@@ -89,7 +130,7 @@ int ExternalCodeEditor::is_editing() {
   return( (pinfo_.dwProcessId != 0) ? 1 : 0 );
 }
 
-// [Static/Local] Terminate_app()'s callback to send WM_CLOSE to a single window. 
+// [Static/Local] Terminate_app()'s callback to send WM_CLOSE to a single window.
 static BOOL CALLBACK terminate_app_enum(HWND hwnd, LPARAM lParam) {
   DWORD dwID;
   GetWindowThreadProcessId(hwnd, &dwID);
@@ -138,24 +179,26 @@ void ExternalCodeEditor::close_editor() {
   // Wait until editor is closed + reaped
   while ( is_editing() ) {
     switch ( reap_editor() ) {
+      case -2:  // no editor running (unlikely to happen)
+        return;
       case -1:  // error
-        fl_alert("Error reaping external editor\n"
-                 "pid=%ld file=%s", long(pinfo_.dwProcessId), filename());
+        fl_alert("Error reaping external editor\npid=%ld file=%s\nOS error message=%s",
+                 long(pinfo_.dwProcessId), filename(), get_ms_errmsg());
         break;
       case 0:   // process still running
-	switch ( fl_choice("Please close external editor\npid=%ld file=%s",
-			   "Force Close",	// button 0
-			   "Closed",		// button 1
-			   0,			// button 2
-			   long(pinfo_.dwProcessId), filename() ) ) {
-	  case 0: 	// Force Close
-	    kill_editor();
-	    continue;
-	  case 1: 	// Closed? try to reap
-	    continue;
-	}
+        switch ( fl_choice("Please close external editor\npid=%ld file=%s",
+                           "Force Close",       // button 0
+                           "Closed",            // button 1
+                           0,                   // button 2
+                           long(pinfo_.dwProcessId), filename() ) ) {
+          case 0:       // Force Close
+            kill_editor();
+            continue;
+          case 1:       // Closed? try to reap
+            continue;
+        }
         break;
-      default:  // process reaped
+      case 1:   // process reaped
         return;
     }
   }
@@ -177,7 +220,7 @@ void ExternalCodeEditor::kill_editor() {
     }
     case 0: {  // success -- process reaped
       DWORD pid = pinfo_.dwProcessId;     // save pid
-      reap_cleanup();
+      reap_cleanup();                     // clears pinfo_
       if ( G_debug )
         printf("*** kill_editor() REAP pid=%ld #open=%ld\n",
                long(pid), long(L_editors_open));
@@ -204,7 +247,8 @@ int ExternalCodeEditor::handle_changes(const char **code, int force) {
   code[0] = 0;
   if ( !is_editing() ) return 0;
   // Sigh, have to open file to get file time/size :/
-  HANDLE fh = CreateFile(filename(),      // file to read
+  utf8_to_wchar(filename(), wbuf);
+  HANDLE fh = CreateFileW(wbuf,           // file to read
                          GENERIC_READ,    // reading only
                          FILE_SHARE_READ, // sharing -- allow read share; just getting file size
                          NULL,            // security
@@ -216,7 +260,7 @@ int ExternalCodeEditor::handle_changes(const char **code, int force) {
   // Get file size
   if ( GetFileSizeEx(fh, &fsize) == 0 ) {
     DWORD err = GetLastError();
-    CloseHandle(fh); 
+    CloseHandle(fh);
     SetLastError(err);  // return error from GetFileSizeEx(), not CloseHandle()
     return -1;
   }
@@ -224,7 +268,7 @@ int ExternalCodeEditor::handle_changes(const char **code, int force) {
   FILETIME ftCreate, ftAccess, ftWrite;
   if ( GetFileTime(fh, &ftCreate, &ftAccess, &ftWrite) == 0 ) {
     DWORD err = GetLastError();
-    CloseHandle(fh); 
+    CloseHandle(fh);
     SetLastError(err);  // return error from GetFileTime(), not CloseHandle()
     return -1;
   }
@@ -237,17 +281,18 @@ int ExternalCodeEditor::handle_changes(const char **code, int force) {
   // Changes? Load file. Be sure to fallthru to CloseHandle()
   int ret = 0;
   if ( changed || force ) {
-    char *buf = (char*)malloc(fsize.QuadPart + 1);
+    DWORD buflen = (DWORD)fsize.QuadPart;
+    char *buf = (char*)malloc((size_t)buflen + 1);
     DWORD count;
-    if ( ReadFile(fh, buf, fsize.QuadPart, &count, 0) == 0 ) {
+    if ( ReadFile(fh, buf, buflen, &count, 0) == 0 ) {
       fl_alert("ERROR: ReadFile() failed for %s: %s",
                filename(), get_ms_errmsg());
       free((void*)buf); buf = 0;
       ret = -1;      // fallthru to CloseHandle()
-    } else if ( count != fsize.QuadPart ) {
+    } else if ( count != buflen ) {
       fl_alert("ERROR: ReadFile() failed for %s:\n"
                "expected %ld bytes, got %ld",
-               filename(), long(fsize.QuadPart), long(count));
+               filename(), long(buflen), long(count));
       free((void*)buf); buf = 0;
       ret = -1;      // fallthru to CloseHandle()
     } else {
@@ -274,7 +319,8 @@ int ExternalCodeEditor::remove_tmpfile() {
   // Filename set? remove (if exists) and zero filename/mtime/size
   if ( is_file(tmpfile) ) {
     if ( G_debug ) printf("Removing tmpfile '%s'\n", tmpfile);
-    if ( DeleteFile(tmpfile) == 0 ) {
+    utf8_to_wchar(tmpfile, wbuf);
+    if (DeleteFileW(wbuf) == 0) {
       fl_alert("WARNING: Can't DeleteFile() '%s': %s", tmpfile, get_ms_errmsg());
       return -1;
     }
@@ -291,9 +337,12 @@ int ExternalCodeEditor::remove_tmpfile() {
 //     Returns pointer to static memory.
 //
 const char* ExternalCodeEditor::tmpdir_name() {
-  char tempdir[100];
-  if (GetTempPath(sizeof(tempdir), tempdir) == 0 ) {
+  wchar_t tempdirW[FL_PATH_MAX+1];
+  char tempdir[FL_PATH_MAX+1];
+  if (GetTempPathW(FL_PATH_MAX, tempdirW) == 0) {
     strcpy(tempdir, "c:\\windows\\temp");      // fallback
+  } else {
+    strcpy(tempdir, wchar_to_utf8(tempdirW, abuf));
   }
   static char dirname[100];
   _snprintf(dirname, sizeof(dirname), "%s.fluid-%ld",
@@ -309,7 +358,8 @@ void ExternalCodeEditor::tmpdir_clear() {
   const char *tmpdir = tmpdir_name();
   if ( is_dir(tmpdir) ) {
     if ( G_debug ) printf("Removing tmpdir '%s'\n", tmpdir);
-    if ( RemoveDirectory(tmpdir) == 0 ) {
+    utf8_to_wchar(tmpdir, wbuf);
+    if ( RemoveDirectoryW(wbuf) == 0 ) {
       fl_alert("WARNING: Can't RemoveDirectory() '%s': %s",
                tmpdir, get_ms_errmsg());
     }
@@ -322,7 +372,8 @@ void ExternalCodeEditor::tmpdir_clear() {
 const char* ExternalCodeEditor::create_tmpdir() {
   const char *dirname = tmpdir_name();
   if ( ! is_dir(dirname) ) {
-    if ( CreateDirectory(dirname,0) == 0 ) {
+    utf8_to_wchar(dirname, wbuf);
+    if (CreateDirectoryW(wbuf, 0) == 0) {
       fl_alert("can't create directory '%s': %s",
         dirname, get_ms_errmsg());
       return NULL;
@@ -338,8 +389,7 @@ const char* ExternalCodeEditor::tmp_filename() {
   static char path[512];
   const char *tmpdir = create_tmpdir();
   if ( !tmpdir ) return 0;
-  extern const char *code_file_name;    // fluid's global
-  const char *ext  = code_file_name;    // e.g. ".cxx"
+  const char *ext  = g_project.code_file_name.c_str();    // e.g. ".cxx"
   _snprintf(path, sizeof(path), "%s\\%p%s", tmpdir, (void*)this, ext);
   path[sizeof(path)-1] = 0;
   return path;
@@ -358,7 +408,8 @@ static int save_file(const char *filename,
   if ( code == 0 ) code = "";   // NULL? write an empty file
   memset(&file_mtime, 0, sizeof(file_mtime));
   memset(&file_size, 0, sizeof(file_size));
-  HANDLE fh = CreateFile(filename,                // filename
+  utf8_to_wchar(filename, wbuf);
+  HANDLE fh = CreateFileW(wbuf,                   // filename
                          GENERIC_WRITE,           // write only
                          0,                       // sharing -- no share during write
                          NULL,                    // security
@@ -371,7 +422,7 @@ static int save_file(const char *filename,
     return(-1);
   }
   // Write the file, being careful to CloseHandle() even on errs
-  DWORD clen = strlen(code);
+  DWORD clen = (DWORD)strlen(code);
   DWORD count = 0;
   int ret = 0;
   if ( WriteFile(fh, code, clen, &count, NULL) == 0 ) {
@@ -410,7 +461,7 @@ int ExternalCodeEditor::start_editor(const char *editor_cmd,
   if ( G_debug ) printf("start_editor() cmd='%s', filename='%s'\n",
                         editor_cmd, filename);
   // Startup info
-  STARTUPINFO sinfo;
+  STARTUPINFOW sinfo;
   memset(&sinfo, 0, sizeof(sinfo));
   sinfo.cb          = sizeof(sinfo);
   sinfo.dwFlags     = 0;
@@ -420,9 +471,10 @@ int ExternalCodeEditor::start_editor(const char *editor_cmd,
   // Command
   char cmd[1024];
   _snprintf(cmd, sizeof(cmd), "%s %s", editor_cmd, filename);
+  utf8_to_wchar(cmd, wbuf);
   // Start editor process
-  if (CreateProcess(NULL,               // app name
-                    (char*)cmd,         // command to exec
+  if (CreateProcessW(NULL,              // app name
+                    wbuf,               // command to exec
                     NULL,               // secure attribs
                     NULL,               // thread secure attribs
                     FALSE,              // handle inheritance
@@ -446,7 +498,7 @@ int ExternalCodeEditor::start_editor(const char *editor_cmd,
 // [Protected] Cleanup after editor reaped:
 //    > Remove tmpfile, zeroes mtime/size/filename
 //    > Close process handles
-//    > Zero out process info
+//    > Zeroes out pinfo_
 //    > Decrease editor count
 //
 void ExternalCodeEditor::reap_cleanup() {
@@ -459,27 +511,31 @@ void ExternalCodeEditor::reap_cleanup() {
 }
 
 // [Public] Try to reap external editor process
+// If 'pid_reaped' not NULL, returns PID of reaped editor.
 // Returns:
 //   -2 -- editor not open
 //   -1 -- WaitForSingleObject() failed (get_ms_errmsg() has reason)
 //    0 -- process still running
-//   >0 -- process finished + reaped (value is pid)
+//    1 -- process finished + reaped ('pid_reaped' has pid), pinfo_ set to 0.
 //         Handles removing tmpfile/zeroing file_mtime/file_size/filename
 //
-DWORD ExternalCodeEditor::reap_editor() {
-  if ( pinfo_.dwProcessId == 0 ) return -2;
-  int err;
+// If return value <=0, 'pid_reaped' is set to zero.
+//
+int ExternalCodeEditor::reap_editor(DWORD *pid_reaped) {
+  if ( pid_reaped ) *pid_reaped = 0;
+  if ( !is_editing() ) return -2;
   DWORD msecs_wait = 50;   // .05 sec
-  switch ( err = WaitForSingleObject(pinfo_.hProcess, msecs_wait) ) {
+  switch ( WaitForSingleObject(pinfo_.hProcess, msecs_wait) ) {
     case WAIT_TIMEOUT: {   // process didn't reap, still running
       return 0;
     }
     case WAIT_OBJECT_0: {  // reaped
-      DWORD pid = pinfo_.dwProcessId;      // save pid
-      reap_cleanup();
+      DWORD wpid = pinfo_.dwProcessId;      // save pid
+      reap_cleanup();                       // clears pinfo_
+      if ( pid_reaped ) *pid_reaped = wpid; // return pid to caller
       if ( G_debug ) printf("*** EDITOR REAPED: pid=%ld #open=%d\n",
-                            long(pid), L_editors_open);
-      return pid;
+                            long(wpid), L_editors_open);
+      return 1;
     }
     case WAIT_FAILED: {    // failed
       return -1;
@@ -488,8 +544,10 @@ DWORD ExternalCodeEditor::reap_editor() {
   return -1;               // any other return unexpected
 }
 
-// [Public] Open external editor using 'editor_cmd' to edit 'code'
+// [Public] Open external editor using 'editor_cmd' to edit 'code'.
+//
 // 'code' contains multiline code to be edited as a temp file.
+// 'code' can be NULL -- edits an empty file if so.
 //
 // Returns:
 //   0 if succeeds
@@ -507,8 +565,10 @@ int ExternalCodeEditor::open_editor(const char *editor_cmd,
   if ( is_file(filename()) ) {
     if ( is_editing() ) {
       // See if editor recently closed but not reaped; try to reap
-      DWORD wpid = reap_editor();
-      switch (wpid) {
+      DWORD wpid;
+      switch ( reap_editor(&wpid) ) {
+        case -2:        // no editor running (unlikely to happen)
+          break;
         case -1:        // wait failed
           fl_alert("ERROR: WaitForSingleObject() failed: %s\nfile='%s', pid=%ld",
             get_ms_errmsg(), filename(), long(pinfo_.dwProcessId));
@@ -517,7 +577,7 @@ int ExternalCodeEditor::open_editor(const char *editor_cmd,
           fl_alert("Editor Already Open\n  file='%s'\n  pid=%ld",
             filename(), long(pinfo_.dwProcessId));
           return 0;
-        default:        // process reaped, wpid is pid reaped
+        case 1:         // process reaped, wpid is pid reaped
           if ( G_debug )
             printf("*** REAPED EXTERNAL EDITOR: PID %ld\n", long(wpid));
           break;        // fall thru to open new editor instance
@@ -570,8 +630,3 @@ void ExternalCodeEditor::set_update_timer_callback(Fl_Timeout_Handler cb) {
 int ExternalCodeEditor::editors_open() {
   return L_editors_open;
 }
-
-#endif /* WIN32 */
-//
-// End of "$Id: ExternalCodeEditor_WIN32.cxx 11879 2016-08-16 21:08:40Z greg.ercolano $".
-//
