@@ -1,6 +1,6 @@
 #define fastarrow_width 16
 #define fastarrow_height 16
-static unsigned char fastarrow_bits[] = {
+static const unsigned char fastarrow_bits[] = {
    0x00, 0x00, 0x00, 0x07, 0xe0, 0x07, 0xfc, 0x03, 0xff, 0xff, 0xfc, 0x03,
    0xe0, 0x07, 0x00, 0x07, 0xe0, 0x00, 0xe0, 0x07, 0xc0, 0x3f, 0xff, 0xff,
    0xc0, 0x3f, 0xe0, 0x07, 0xe0, 0x00, 0x00, 0x00};
