@@ -1,19 +1,17 @@
 //
-// "$Id: color_chooser.cxx 10694 2015-04-09 20:10:25Z AlbrechtS $"
-//
 // Color chooser test program for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2010 by Bill Spitzak and others.
+// Copyright 1998-2018 by Bill Spitzak and others.
 //
 // This library is free software. Distribution and use rights are outlined in
 // the file "COPYING" which should have been included with this file.  If this
 // file is missing or damaged, see the license at:
 //
-//     http://www.fltk.org/COPYING.php
+//     https://www.fltk.org/COPYING.php
 //
-// Please report all bugs and problems on the following page:
+// Please see the following page on how to report bugs and issues:
 //
-//     http://www.fltk.org/str.php
+//     https://www.fltk.org/bugs.php
 //
 
 #include <FL/Fl.H>
@@ -23,14 +21,12 @@
 #include <FL/fl_show_colormap.H>
 #include <FL/Fl_Color_Chooser.H>
 #include <FL/Fl_Image.H>
-#include <FL/x.H>
+#include <FL/platform.H>
 #include <FL/fl_draw.H>
 
 #include <stdlib.h>
 #include <stdio.h>
-#if !defined(WIN32) && !defined(__APPLE__)
 #include "list_visuals.cxx"
-#endif
 
 int width = 100;
 int height = 100;
@@ -45,14 +41,14 @@ void make_image() {
     for (int x = 0; x < width; x++) {
       double X = double(x)/(width-1);
       *p++ = uchar(255*((1-X)*(1-Y))); // red in upper-left
-      *p++ = uchar(255*((1-X)*Y));	// green in lower-left
-      *p++ = uchar(255*(X*Y));	// blue in lower-right
+      *p++ = uchar(255*((1-X)*Y));     // green in lower-left
+      *p++ = uchar(255*(X*Y));         // blue in lower-right
     }
   }
 }
 
 class Pens : public Fl_Box {
-  void draw();
+  void draw() FL_OVERRIDE;
 public:
   Pens(int X, int Y, int W, int H, const char* L)
   : Fl_Box(X,Y,W,H,L) {}
@@ -114,7 +110,7 @@ int main(int argc, char ** argv) {
            " - : default visual\n"
            " r : call Fl::visual(FL_RGB)\n"
            " c : call Fl::own_colormap()\n",argv[0]);
-#if !defined(WIN32) && !defined(__APPLE__)
+#ifdef FLTK_USE_X11
     printf(" # : use this visual with an empty colormap:\n");
     list_visuals();
 #endif
@@ -127,7 +123,7 @@ int main(int argc, char ** argv) {
     } else if (argv[i][0] == 'c') {
       Fl::own_colormap();
     } else if (argv[i][0] != '-') {
-#if !defined(WIN32) && !defined(__APPLE__)
+#ifdef FLTK_USE_X11
       int visid = atoi(argv[i]);
       fl_open_display();
       XVisualInfo templt; int num;
@@ -138,14 +134,10 @@ int main(int argc, char ** argv) {
                                     fl_visual->visual, AllocNone);
       fl_xpixel(FL_BLACK); // make sure black is allocated
 #else
-      Fl::fatal("Visual id's not supported on MSWindows or MacOS.");
+      Fl::fatal("Visual id's not supported on Windows or MacOS.");
 #endif
     }
   }
   window.show(argc,argv);
   return Fl::run();
 }
-
-//
-// End of "$Id: color_chooser.cxx 10694 2015-04-09 20:10:25Z AlbrechtS $".
-//
